// -------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

float4 MaskColor : COLOR <string uiname="Mask Color";>  = {  1,    1,    1,    1.00000  };
float4 TargetColor : COLOR <string uiname="Target Color";>  = {  1,    1,    1,    1.00000  };
//pin for threshold
float tolerance <string uiname="Tolerance";>  = 0.10;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos : POSITION;
    float4 TexCd : TEXCOORD0;
};
// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS(
    float4 Pos : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(Pos, tWVP);

    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}

// -------------------------------------------------------------------------
// PIXELSHADERS:
// -------------------------------------------------------------------------


float4 changecolor(vs2ps In): COLOR
{
    //texture lookup to access the pixels of the texture
    float4 col = tex2D(Samp, In.TexCd);

    float low_r_threshold = MaskColor.r - tolerance;
    float hi_r_threshold = MaskColor.r + tolerance;
    float low_g_threshold = MaskColor.g - tolerance;
    float hi_g_threshold = MaskColor.g + tolerance;
    float low_b_threshold = MaskColor.b - tolerance;
    float hi_b_threshold = MaskColor.b + tolerance;

    if (col.r > low_r_threshold && col.r < hi_r_threshold &&
        col.g > low_g_threshold && col.g < hi_g_threshold &&
        col.b > low_b_threshold && col.b < hi_b_threshold)
    {
       col = TargetColor;
    }
    return col;
}

// -------------------------------------------------------------------------
// TECHNIQUES:
// -------------------------------------------------------------------------

technique ChangeColor //name for the technique pin
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 changecolor();
    }
}
